package ucot.iter1;

import java.util.List;
import java.util.Set;

import ucot.iter1.input.ParsedData;
import ucot.iter1.input.Sentence;
import ucot.iter1.input.Word;
import ucot.iter1.model.AnalyzeModel;
import ucot.iter1.model.Entity;
import ucot.iter1.model.Method;
import ucot.iter1.model.Relation;
import ucot.iter1.model.RelationType;
/**
 * Abbotts Heuristic is a way to etsimated which
 * parts of the speech are relevant and in which
 * way when doing object analyze.
 * 
 * @author pajumasu
 *
 */
public class AbbottsHeuristic {
    
    /**
     * Method creates AnalyzeModel from the 
     * parsed data.
     * @param data
     * @return
     */
    public AnalyzeModel getModel(ParsedData data) {
        AnalyzeModel model = new AnalyzeModel();
        
        // Loop through all of the sences.
        for (Sentence s : data.getSentences()) {
            
            // Loop through all the words.
            for (Word word : s.getWords()) {
                
                // Basic information from the word.
                String basicForm = word.getBasicForm();
                String wordClass = word.getWordClass();
                
                if (wordClass.equals("noun")) {
                    // Word is noun. So its is probably an entity.
                    model.addItem(new Entity(basicForm));
                } else if (wordClass.equals("verb")) {
                    // Word is verb. So it is probably a operation
                    // between entities.
                    
                    // Check if there exist information 
                    // about the structure of the sentence. 
                    Word subject = word.getLinked("subject");
                    Word object = word.getLinked("object");
                    
                    if (null != subject && null != object) {
                        // There is structural information
                        // so we can create entity to entity
                        // relation.
                        
                        Entity subjectE = new Entity(subject.getBasicForm());
                        Entity objectE = new Entity(object.getBasicForm());
                        
                        // Create relation called by the name of the verb.
                        Relation relation = new Relation(subjectE, word
                                .getBasicForm(), objectE);
                        
                        model.addRelation(relation);
                    }
                    
                    if (null != subject){
                        // We know the subject
                        // so we can add "method" to the entity.
                        
                        Method method = null;
                        
                        
                        Entity subjectE = new Entity(subject.getBasicForm());
                        
                        
                        // Code blocks finds equally named methods for the
                        // entity.
                        Set<Relation> rels = model.getRelationsFrom(subjectE,"method");
                        for(Relation relation : rels) {
                            if (relation.getTo().getName().equals(word.getBasicForm())){
                                method = (Method) relation.getTo();
                                break;
                            }
                        }
                        
                        
                        if (null == method)  {
                            // No method was found so create one.
                            // If methdo was found then there is no
                            // need to add anything.
                            method = new Method(word.getBasicForm());
                            Relation relation = new Relation(subjectE, RelationType.CONTAINS, method);
                            model.addRelation(relation);
                        }
                        
                        
                    }
                    
                }
            }
        }
        
        return model;
    }
    
    
}
